/*
 * main.c
 * lcdload
 * 
 * Shows the current load on the Fujitsu C-4110 lcd
 * To be used together with apanel
 * 
 * See GPL for license
 * (c) Copyright by Willem van Engen <wvengen@stack.nl>
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/resource.h>
#include "lcdload.h"

int main(int argc,char* argv[]) {
  int daemonmode=0;
  int sampleno=0;
  long int interval=1000;
  int load;
  int wrap=0;
  double loads[3];
  FILE *panel;
  char ch;
  
  /* Parse cmdline */
  while ((ch=getopt(argc,argv,"dhi:t:w")) != -1) {
    switch(ch) {
    case 'd':
      daemonmode=1;
      break;

    case 'i':
      interval=atoi(optarg)*1000;
      if (interval=0) {
        fprintf(stderr,"An interval of 0 will load your system too heavily\n");
	exit(1);
      }
      break;

    case 't':
      switch(atoi(optarg)) {
      case 1:
        sampleno=0;
	interval=1000;
	break;

      case 5:
	sampleno=1;
	interval=5000;
	break;

      case 15:
	sampleno=2;
	interval=15000;
	break;

      default:
	fprintf(stderr,"Valid time periods are 1, 5 and 15 minutes\n");
	exit(1);
      }
      break;

    case 'w':
      wrap=1;
      break;

    case 'h':
      printf(HELPMSG);
      return 0;

    default:
      printf("Unknown option: %c. Use -h for help.\n",ch);
    }
  }

  if ((panel=fopen("/dev/apanel","w"))==NULL) {
    fprintf(stderr,"Could not open /dev/apanel application panel device. "
		   "Is this a Fujitsu C-4110?\n");
    exit(1);
  }

  /* Renice myself */
  setpriority(PRIO_PROCESS,0,19);

  /* become daemon if requested */
  if (daemonmode==1) {
    if (daemon(NULL,NULL)==-1) {
      fprintf(stderr,"Could not become daemon\n");
      return 1;
    }
  }

  while (1) {
    if (getloadavg(loads,3)!=-1) {
      load=(int)(100*loads[sampleno]);
      if (load==0) load=1;
      if (load>99 && wrap==1) load=99;
      fprintf(panel,"d%d",load);
      fflush(panel);
    }
    usleep(interval);
  }
  return 0;
}
